<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00004004ReajusteContratoFormacaoPrecoService;

class Sm00004004ReajusteContratoFormacaoPrecoController extends SMCrud
{
    protected $dataSourceName = 'fct_calcformprecserv_widget';
    protected $reajusteContratoService;
     
    public function __construct ($crudService, $environment, Sm00004004ReajusteContratoFormacaoPrecoService $reajusteContratoService) {
        parent::__construct($crudService, $environment);
        $this->reajusteContratoService = $reajusteContratoService;
    }
   
    /**
    *  Função que salva reajuste de acordo com a rows selecionada
    */
    
     public function saveReajuste(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $rows                 = $row["ROWS"];
            $idtpajuste           = $row["IDTPAJUSTE"];
            $vrpercdescajuste     = $row["VRPERCDESCAJUSTE"];
            $nrorg                = $this->environment->getNrOrg();
            $cdoperador           = $this->environment->getCdOperador();
            
            $nrseqcontratos = array();
            $valororiginais = array();
            foreach ($rows as $row) {
                if(!in_array($row["NRSEQCONTRATO"],$nrseqcontratos)){
                    array_push($nrseqcontratos, $row["NRSEQCONTRATO"]);
                }
                if(!in_array($row["VALORORIGINAL"],$valororiginais)){
                array_push($valororiginais, $row["VALORORIGINAL"]);
                }
            }
            
            $this->reajusteContratoService->selecionaUltimoReajuste($nrseqcontratos, $valororiginais, $idtpajuste, $vrpercdescajuste, $nrorg, $cdoperador);  
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
    *  Função que salva reajuste de elemento na tabela temporária
    */
    
     public function saveReajusteElemento(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $escolhidos = $row["ESCOLHIDOS"];
            $tabela = $row["TABELA"];
            $idtpajuste = $row["IDTPAJUSTEELEM"];
            $vrpercdescajuste = $row["VRPERCDESCAJUSTEELEM"];
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->reajusteContratoService->saveReajusteElemento($nrformprecoservico, $escolhidos, $tabela, $idtpajuste, $vrpercdescajuste, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador);  
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
    *  Função que salva reajuste de elemento que estava na tabela temporária para a tabela permanente
    */
    
    public function aplicarReajuste(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $nrreajuscontformprec = $row["NRREAJUSCONTFORMPREC"];
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $valorpai = $row["VALORPAI"];
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            if(!$valorpai){
                $tabela = 1;
                }else{
                    $tabela = 2;
                    }
            
            $aplicado = $this->reajusteContratoService->aplicarReajuste($nrformprecoservico, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador, $nrreajuscontformprec, $tabela); 
            $response->addDataSet(
                new DataSet(
                    "APLICADO", array(
                        "aplicado" => $aplicado
                    )
                )
            );
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
    *  Função que deleta reajuste que esta na tabela temporária
    */
    
     public function removerAjusteTemp(Request $request, Response $response){
        try {
            $nrorg = $this->environment->getNrOrg();
            $this->reajusteContratoService->removerAjusteTemp($nrorg); 
           
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
    *  Função que salva multiplos reajustes de preço 
    */
    
    public function saveReajusteAutomatico(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $formacoes = $row["FORMACOES"];
            $idtpajuste = $row["IDTPAJUSTE"];
            $vrpercdescajuste = $row["VRPERCDESCAJUSTE"];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            foreach($formacoes as $formacao){
                $nrformprecoservico = $formacao["NRFORMPRECOSERVICO"];
                $nrreajuscontformprec = $formacao["NRREAJUSCONTFORMPREC"];
                $codigo = $formacao["CODIGO"];
                $dados = $formacao["DADOS"];
                $nrseqcontrato = $formacao["NRSEQCONTRATO"];
                $nritemocontrato = $formacao["NRITEMOCONTRATO"];
                $formacaoreajuste = array();
                
                foreach($dados as $row){
                    array_push($formacaoreajuste, $row["FORMACAOREAJUSTE"]);
                }
       
             $this->reajusteContratoService->saveReajusteAutomatico($nrreajuscontformprec, $nrformprecoservico, $codigo, $formacaoreajuste, $idtpajuste, $vrpercdescajuste, $nrseqcontrato, $nritemocontrato, $nrorg, $cdoperador);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
    *  Buica pelo o ultimo reajuste de cada contrato
    */
    
    public function buscarUltimosReajustes(Request $request, Response $response){
        try {
            
            $nrorg = $this->environment->getNrOrg();
            $ultimos = array();

           $rows = $this->reajusteContratoService->buscarFormacoesContratos($nrorg);
            foreach ($rows as $row) {
                $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
                $nrseqcontrato = $row["NRSEQCONTRATO"];
                $ultimo = $this->reajusteContratoService->UltimosReajustesPorContrato($nrformprecoservico,$nrseqcontrato,$nrorg);
                        array_push($ultimos, $ultimo[0]["MAX(NRREAJUSCONTFORMPREC)"]);
                    }
            $response->addDataSet(new DataSet("dados", $ultimos));

            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    /**
    *  Buica pelo o ultimo reajuste de cada contrato
    */
    
    public function buscarNumeroFormacoes(Request $request, Response $response){
        try {
            
            $nrorg = $this->environment->getNrOrg();
            $ultimos = array();
            $formacoes = array();
            
           $rows = $this->reajusteContratoService->buscarFormacoesContratos($nrorg);
            foreach ($rows as $row) {
                array_push($formacoes,$row["NRFORMPRECOSERVICO"]);
            }
            foreach ($rows as $row) {
                $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
                $nrseqcontrato = $row["NRSEQCONTRATO"];
                $ultimo = $this->reajusteContratoService->UltimosReajustesPorContrato($nrformprecoservico,$nrseqcontrato,$nrorg);
                        array_push($ultimos, $ultimo[0]["MAX(NRREAJUSCONTFORMPREC)"]);
            }
            $dados = array_merge($formacoes, $ultimos);
            $response->addDataSet(new DataSet("dados", $dados));

            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    
    
    /* -------------------------  --------------------------- ---------------------*/
    
    
         /**
     *  A função "recalculoElemento" realiza o recalculo dos elementos.
     */
    
    public function recalculoElemento(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrcalcreajformpre = $row['NRCALCREAJFORMPRE'];
        $this->reajusteContratoService->recalculoElemento($nrcalcreajformpre);
    }
    
    
    /**
     *  A função "recalculoElemento" realiza o recalculo de toda a árvore dos elementos.
     */
    
    public function recalcularArvore(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrreajuscontformprec = $row['NRREAJUSCONTFORMPREC'];
        $this->reajusteContratoService->recalcularArvore($nrreajuscontformprec);
    }

}
?>